@@ -24,6 +24,9 @@ import pickle
 from functools import lru_cache
 from typing import Tuple, Dict, List, Union, Set, Callable
 
+from matplotlib import cm
+from matplotlib.colors import Normalize, to_hex
+
 np.seterr(all="raise")
 
 
@@ -227,9 +230,12 @@ class CollapsedTree:
             dlogfdq_result = 2 / q
         else:
             if m >= 1:
-                neighbor_ll_genotype, (
-                    neighbor_dlogfdp,
-                    neighbor_dlogfdq,
+                (
+                    neighbor_ll_genotype,
+                    (
+                        neighbor_dlogfdp,
+                        neighbor_dlogfdq,
+                    ),
                 ) = CollapsedTree._ll_genotype(c, m - 1, p, q)
                 logf_result = (
                     np.log(2)
@@ -334,7 +340,7 @@ class CollapsedTree:
         Returns:
             Tuple :math:`(p, q)` with estimated branching probability and estimated mutation probability
         """
-        return _mle_helper(self.ll)
+        return _mle_helper(self.ll, **kwargs)
 
     def simulate(self, p: np.float64, q: np.float64, root: bool = True):
         r"""Simulate a collapsed tree as an infinite type Galton-Watson process
@@ -373,6 +379,9 @@ class CollapsedTree:
     def render(
         self,
         outfile: str,
+        scale: float = None,
+        branch_margin: float = 0,
+        node_size: float = None,
         idlabel: bool = False,
         colormap: Dict = None,
         frame: int = None,
@@ -386,7 +395,11 @@ class CollapsedTree:
 
         Args:
             outfile: file name to render to, filetype inferred from suffix, .svg for color
+            scale: branch length scale in pixels (set automatically if ``None``)
+            branch_margin: additional leaf branch separation margin, in pixels, to scale tree width
+            node_size: size of nodes in pixels (set according to abundance if ``None``)
             idlabel: label nodes with seq ids, and write sequences of all nodes to a fasta file with same base name as ``outfile``
+            colormap: dictionary mapping node names to color names or to dictionaries of color frequencies
             frame: coding frame for annotating amino acid substitutions
             position_map: mapping of position names for sequence indices, to be used with substitution annotations and the ``frame`` argument
             chain_split: if sequences are a concatenation two gene sequences, this is the index at which the 2nd one starts (requires ``frame`` and ``frame2`` arguments)
@@ -403,36 +416,50 @@ class CollapsedTree:
             else:
                 circle_color = colormap[node.name]
             text_color = "black"
-            if isinstance(circle_color, str):
-                C = ete3.CircleFace(
-                    radius=max(3, 10 * np.sqrt(node.abundance)),
-                    color=circle_color,
-                    label={"text": str(node.abundance), "color": text_color}
-                    if node.abundance > 0
-                    else None,
-                )
-                C.rotation = -90
-                C.hz_align = 1
-                ete3.faces.add_face_to_node(C, node, 0)
-            else:
-                P = ete3.PieChartFace(
-                    [100 * x / node.abundance for x in circle_color.values()],
-                    2 * 10 * np.sqrt(node.abundance),
-                    2 * 10 * np.sqrt(node.abundance),
-                    colors=[
-                        (color if color != "None" else "lightgray")
-                        for color in list(circle_color.keys())
-                    ],
-                    line_color=None,
-                )
-                T = ete3.TextFace(
-                    " ".join([str(x) for x in list(circle_color.values())]),
-                    tight_text=True,
-                )
-                T.hz_align = 1
+
+            if node_size is None:
+                node_size2 = max(1, 10 * np.sqrt(node.abundance))
+                if isinstance(circle_color, str):
+                    C = ete3.CircleFace(
+                        radius=node_size2,
+                        color=circle_color,
+                        label={"text": str(node.abundance), "color": text_color}
+                        if node.abundance > 0
+                        else None,
+                    )
+                    C.rotation = -90
+                    C.hz_align = 1
+                    ete3.faces.add_face_to_node(C, node, 0)
+                else:
+                    P = ete3.PieChartFace(
+                        [100 * x / node.abundance for x in circle_color.values()],
+                        2 * node_size2,
+                        2 * node_size2,
+                        colors=[
+                            (color if color != "None" else "lightgray")
+                            for color in list(circle_color.keys())
+                        ],
+                        line_color=None,
+                    )
+                    T = ete3.TextFace(
+                        " ".join([str(x) for x in list(circle_color.values())]),
+                        tight_text=True,
+                    )
+                    T.hz_align = 1
+                    T.rotation = -90
+                    ete3.faces.add_face_to_node(P, node, 0, position="branch-right")
+                    ete3.faces.add_face_to_node(T, node, 1, position="branch-right")
+            elif node.abundance:
+                T = ete3.TextFace(node.abundance)
                 T.rotation = -90
-                ete3.faces.add_face_to_node(P, node, 0, position="branch-right")
-                ete3.faces.add_face_to_node(T, node, 1, position="branch-right")
+                T.hz_align = 1
+                ete3.faces.add_face_to_node(
+                    T,
+                    node,
+                    0,
+                    position="branch-top",
+                )
+
             if idlabel:
                 T = ete3.TextFace(node.name, tight_text=True, fsize=6)
                 T.rotation = -90
@@ -441,16 +468,25 @@ class CollapsedTree:
                     T,
                     node,
                     1 if isinstance(circle_color, str) else 2,
-                    position="branch-right",
+                    position="branch-bottom",
                 )
 
         # we render on a copy, so faces are not permanent
         tree_copy = self.tree.copy(method="deepcopy")
         for node in tree_copy.traverse():
             nstyle = ete3.NodeStyle()
-            nstyle["size"] = 0
+            if colormap is None or node.name not in colormap:
+                nstyle["fgcolor"] = "lightgray"
+            else:
+                nstyle["fgcolor"] = colormap[node.name]
+            if node_size is not None:
+                nstyle["size"] = node_size
+            else:
+                nstyle["size"] = 0
             if node.up is not None:
-                if set(node.sequence.upper()) == set("ACGT"):
+                if "sequence" in tree_copy.features and set(
+                    node.sequence.upper()
+                ) == set("ACGT"):
                     if frame is not None:
                         if chain_split is not None and frame2 is None:
                             raise ValueError(
@@ -515,14 +551,15 @@ class CollapsedTree:
             node.set_style(nstyle)
 
         ts = ete3.TreeStyle()
+        ts.scale = scale
+        ts.branch_vertical_margin = branch_margin
         ts.show_leaf_name = False
         ts.rotation = 90
         ts.draw_aligned_faces_as_table = False
         ts.allow_face_overlap = True
         ts.layout_fn = my_layout
-        ts.show_scale = False
+        ts.show_scale = True
         ts.show_branch_support = show_support
-        tree_copy.render(outfile, tree_style=ts)
         # if we labelled seqs, let's also write the alignment out so we have
         # the sequences (including of internal nodes)
         if idlabel:
@@ -538,6 +575,40 @@ class CollapsedTree:
             AlignIO.write(
                 aln, open(os.path.splitext(outfile)[0] + ".fasta", "w"), "fasta"
             )
+        return tree_copy.render(outfile, tree_style=ts)
+
+    def feature_colormap(
+        self,
+        feature: str,
+        cmap: str = "viridis",
+        vmin: float = None,
+        vmax: float = None,
+    ) -> Dict[str, str]:
+        r"""Generate a colormap based on a continuous tree feature.
+
+        Args:
+            feature: feature name (all nodes in tree attribute must have this feature)
+            cmap: any matplotlib color palette: https://matplotlib.org/stable/gallery/color/colormap_reference.html
+            vmin: minimum value for colormap (default to minimum of the feature over the tree)
+            vmax: maximum value for colormap (default to maximum of the feature over the tree)
+
+        Returns:
+            Dictionary of node names to hex color strings, which may be used as the colormap in :meth:`gctree.CollapsedTree.render`
+        """
+        cmap = cm.get_cmap(cmap)
+
+        if vmin is None:
+            vmin = np.nanmin([getattr(node, feature) for node in self.tree.traverse()])
+        if vmax is None:
+            vmax = np.nanmax([getattr(node, feature) for node in self.tree.traverse()])
+
+        # define the minimum and maximum values for our colormap
+        norm = Normalize(vmin=vmin, vmax=vmax)
+
+        return {
+            node.name: to_hex(cmap(norm(getattr(node, feature))))
+            for node in self.tree.traverse()
+        }
 
     def write(self, file_name: str):
         r"""Serialize to pickle file.
@@ -721,6 +792,56 @@ class CollapsedTree:
                     compatibility_ += weights[i] if weights is not None else 1
             node.support = compatibility_ if compatibility else support
 
+    def local_branching(self, tau=1, tau0=0.1):
+        r"""Add local branching statistics (Neher et al. 2014) as tree node
+        features to the ETE tree attribute.
+        After execution, all nodes will have new features ``LBI``
+        (local branching index) and ``LBR`` (local branching ratio, below Vs
+        above the node)
+
+        Args:
+            tau: decay timescale for exponential filter
+            tau0: effective branch length for branches with zero mutations
+        """
+        # the fixed integral contribution for clonal cells indicated by abundance annotations
+        clone_contribution = tau * (1 - np.exp(-tau0 / tau))
+
+        # post-order traversal to populate downward integral for each node
+        for node in self.tree.traverse(strategy="postorder"):
+            if node.is_leaf():
+                node.add_feature(
+                    "LB_down",
+                    node.abundance * clone_contribution if node.abundance > 1 else 0,
+                )
+            else:
+                node.add_feature(
+                    "LB_down",
+                    node.abundance * clone_contribution
+                    + sum(
+                        tau * (1 - np.exp(-child.dist / tau))
+                        + np.exp(-child.dist / tau) * child.LB_down
+                        for child in node.children
+                    ),
+                )
+
+        # pre-order traversal to populate upward integral for each node
+        for node in self.tree.traverse(strategy="preorder"):
+            if node.is_root():
+                node.add_feature("LB_up", 0)
+            else:
+                node.add_feature(
+                    "LB_up",
+                    tau * (1 - np.exp(-node.dist / tau))
+                    + np.exp(-node.dist / tau) * (node.up.LB_up + node.up.LB_down),
+                )
+
+        # finally, compute LBI (LBR) as the sum (ratio) of upward and downward integrals at each node
+        for node in self.tree.traverse():
+            node.add_feature("LBI", node.LB_down + node.LB_up)
+            node.add_feature(
+                "LBR", node.LB_down / node.LB_up if not node.is_root() else np.nan
+            )
+
 
 class CollapsedForest:
     r"""A collection of :class:`CollapsedTree`
@@ -826,7 +947,7 @@ class CollapsedForest:
             return ls.sum(), grad_ls.sum(axis=0)
 
     def mle(self, **kwargs) -> Tuple[np.float64, np.float64]:
-        return _mle_helper(self.ll)
+        return _mle_helper(self.ll, **kwargs)
 
     mle.__doc__ = CollapsedTree.mle.__doc__
 
